#ifndef GINAC_JIT_CONTEXT_HPP
#define GINAC_JIT_CONTEXT_HPP
#include <map>
#include <string>
#include "ginac/ex.h"

typedef double (*jit_fptr_1)(double);
typedef double (*jit_fptr_2)(double, double);
typedef double (*jit_fptr_3)(double, double, double);
typedef double (*jit_fptr_4)(double, double, double, double);
typedef double (*jit_fptr_any)(double*, int);

namespace  GiNaC
{

struct jit_context_priv;
class jit_context
{
	jit_context_priv* impl;
public:

	jit_context();
	~jit_context();
	/**
	 * @brief Compile expression into LLVM IR
	 *
	 * @param e expression to compile
	 * @param fname name of native function to create
	 * @param vars_ symbols denoting function arguments
	 * @param safe Ask LLVM to run basic sanity checks on function.
	 *        Switched off by default since these checks are way too
	 *        expensive for any real-life expression.
	 *
	 * Substituting floating-point values for the arguments (@a vars_)
	 * in the expression @a e must yeld a real number. Any functions
	 * referenced by @e must be either
	 * - present in the C runtime library or other libraries linked
	 *   into the main binary
	 * - previously compiled in the same jit_context
	 */
	void operator()(const GiNaC::ex& e,
			const std::string& function_name,
			const GiNaC::exvector& vars,
			const bool safe = false);

	void save(const std::string& file_name);

	/**
	 * @brief Compile previously converted expression into a native code
	 */
	void* get_native();
};

}

#endif // GINAC_JIT_CONTEXT_HPP

